/// <reference types="express" />
/// <reference types="express-session" />
import { RequestHandler, Router } from 'express';
export declare const setLoggerDisplay: (display: boolean) => void;
export declare type StartupFunction = (config: VcmsOptions) => Promise<VcmsOptions> | VcmsOptions;
export declare const defaultOptions: VcmsOptions;
export declare function getConfig(startupConfigScriptPath?: string, configFilepath?: string): Promise<VcmsOptions>;
export declare type Static = {
    route: string | RegExp;
    serve: string;
};
export interface VcmsOptions {
    node_env: string;
    port: number;
    local_hostname: string;
    http2_required: boolean;
    http2_key?: string;
    http2_cert?: string;
    database_required: boolean;
    db_type?: string;
    db_host?: string;
    db_port?: number;
    db_name?: string;
    db_user?: string;
    db_password?: string;
    session_required: boolean;
    redis_host?: string;
    session_cookie_domain?: string;
    configFilepath?: string;
    routers?: {
        [base: string]: Router | RequestHandler;
    };
    initSessionFunction?: (session: Express.Session) => void;
    static?: string;
    statics?: Static[];
    middlewares?: RequestHandler[];
}
export interface ConfigFileOptions extends ConfigFileOptionsBase {
    prod: ConfigFileOptionsBase;
    dev: ConfigFileOptionsBase;
    test: ConfigFileOptionsBase;
}
export interface ConfigFileOptionsBase {
    port?: number;
    'local-hostname'?: string;
    http2?: boolean;
    'http2-key'?: string;
    'http2-cert'?: string;
    database?: boolean;
    'db-type'?: string;
    'db-host'?: string;
    'db-port'?: string;
    'db-name'?: string;
    'db-user'?: string;
    'db-password'?: string;
    session?: boolean;
    'redis-host'?: string;
    'session-cookie-domain'?: string;
    static?: string;
    'statics'?: Static[];
}
export interface CommandLineOptions {
    port?: number;
    'local-hostname'?: string;
    http2?: boolean;
    'http2-key'?: string;
    'http2-cert'?: string;
    'enable-database'?: boolean;
    'db-port'?: number;
    'db-name'?: string;
    'db-user'?: string;
    'redis-host'?: string;
    'session-cookie-domain'?: string;
    static?: string;
}
