"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = require("crypto");
const Knex = require("knex");
const objection_1 = require("objection");
const readline_sync_1 = require("readline-sync");
const logging_1 = require("./logging");
const logger = new logging_1.Logger('database');
function getDatabase(config) {
    return __awaiter(this, void 0, void 0, function* () {
        logger.log('initialising database...');
        let knexDialect;
        // specific options
        switch (config.db_type) {
            case 'pg':
                knexDialect = 'pg';
        }
        if (!config.db_user) {
            const error = 'No Database User specified !';
            logger.error(error);
            throw new Error(error);
        }
        /* PASSWORD */
        let password = config.db_password;
        if (!config.db_password) {
            password = readline_sync_1.question(`Enter Database password for ${config.db_user}: `, { hideEchoBack: true });
        }
        // if the password in the configuration file is a hash
        if (password.substr(0, 5) === '{SHA}') {
            logger.error('The password in the configuration file is encrypted.');
            password = readline_sync_1.question(`Enter Database password for ${config.db_user}: `, { hideEchoBack: true });
            const sha1 = crypto_1.createHash('sha1');
            sha1.update(password);
            if (sha1.digest('hex') !== config.db_password.substr(5)) {
                const err = new Error('Password for database is incorrect');
                err.name = 'database';
                throw err;
            }
        }
        /* DATABASE NAME */
        let dbname = config.db_name;
        if (!dbname) {
            logger.info(`No Database Name specified, using default (resolved to ${config.db_user})`);
            dbname = config.db_user;
        }
        // database
        const database = Knex({
            client: knexDialect,
            connection: {
                host: config.db_host,
                port: config.db_port,
                user: config.db_user,
                password: password,
                database: dbname,
            },
            pool: {
                min: 0,
                max: 1,
                afterCreate: function (conn, done) {
                    logger.info('a new pool was created');
                    done();
                }
            }
        });
        // testing the connection now
        try {
            yield testDatabaseConnection(database);
        }
        catch (e) {
            logger.error(e.message);
            throw e;
        }
        // this line makes new made models automatically be mapped to the database
        objection_1.Model.knex(database);
        logger.success('Database ready.');
        return database;
    });
}
exports.getDatabase = getDatabase;
function testDatabaseConnection(database) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const r = yield database.raw('select 1+1 as result');
        }
        catch (e) {
            throw new Error(e.message);
        }
    });
}
exports.testDatabaseConnection = testDatabaseConnection;
//# sourceMappingURL=database.js.map