"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const config_1 = require("../config");
const Role_1 = require("../models/Role");
const server_1 = require("../server");
function getConfig(args = [], startupConfigPath = null, vcmsFilepath = null) {
    return __awaiter(this, void 0, void 0, function* () {
        /* save the context */
        const originalArgv = process.argv;
        const originalNODE_ENV = process.env.NODE_ENV;
        /* set new context */
        process.argv = ['node', 'app'].concat(args);
        if (!process.env.NODE_ENV) {
            process.env.NODE_ENV = 'test';
        }
        const config = yield config_1.getConfig(startupConfigPath, vcmsFilepath);
        /* restore original context */
        process.argv = originalArgv;
        if (!originalNODE_ENV) {
            delete process.env.NODE_ENV;
        }
        else {
            process.env.NODE_ENV = originalNODE_ENV;
        }
        return config;
    });
}
exports.getConfig = getConfig;
function resetDatabase(database, scripts) {
    return __awaiter(this, void 0, void 0, function* () {
        const sql = scripts.map(f => fs_1.readFileSync(f).toString()).join(';');
        yield database.raw(sql);
    });
}
exports.resetDatabase = resetDatabase;
function accessAsGuest(struct) {
    return __awaiter(this, void 0, void 0, function* () {
        struct.config.middlewares = [(req, res, next) => {
                req.session.user.logged = false;
                req.session.user.roles = [new Role_1.default('GUEST')];
                next();
            }];
        // closing connections before making a new structure
        yield server_1.destroyStructure(struct);
        struct = yield server_1.getStructure(struct.config);
        return struct;
    });
}
exports.accessAsGuest = accessAsGuest;
function accessAsUser(id = 1, struct) {
    return __awaiter(this, void 0, void 0, function* () {
        struct.config.middlewares = [(req, res, next) => {
                req.session.user.logged = true;
                req.session.user.roles = [new Role_1.default('USER')];
                req.session.user.id = id;
                next();
            }];
        // closing connections before making a new structure
        yield server_1.destroyStructure(struct);
        struct = yield server_1.getStructure(struct.config);
        return struct;
    });
}
exports.accessAsUser = accessAsUser;
function accessAsAdmin(struct) {
    return __awaiter(this, void 0, void 0, function* () {
        struct.config.middlewares = [(req, res, next) => {
                req.session.user.logged = true;
                req.session.user.roles = [new Role_1.default('USER'), new Role_1.default('ADMIN')];
                req.session.user.id = 1414; // fake id
                next();
            }];
        // closing connections before making a new structure
        yield server_1.destroyStructure(struct);
        struct = yield server_1.getStructure(struct.config);
        return struct;
    });
}
exports.accessAsAdmin = accessAsAdmin;
//# sourceMappingURL=util.js.map